# Generated by roxygen2: do not edit by hand

S3method(plot,mvn)
S3method(summary,mvn)
export(arw_adjustment)
export(descriptives)
export(doornik_hansen)
export(energy)
export(hw)
export(hz)
export(impute_missing)
export(mardia)
export(multivariate_diagnostic_plot)
export(mv_outlier)
export(mvn)
export(power_transform)
export(royston)
export(run_mvn_app)
export(test_univariate_normality)
export(univariate_diagnostic_plot)
importFrom(MASS,cov.mcd)
importFrom(MASS,ginv)
importFrom(MASS,kde2d)
importFrom(boot,boot)
importFrom(car,powerTransform)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_h1)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,reframe)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(energy,mvnorm.e)
importFrom(energy,mvnorm.etest)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,annotate)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,unit)
importFrom(ggplot2,vars)
importFrom(graphics,abline)
importFrom(graphics,boxplot)
importFrom(graphics,contour)
importFrom(graphics,curve)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,persp)
importFrom(graphics,plot)
importFrom(graphics,text)
importFrom(methods,new)
importFrom(mice,complete)
importFrom(mice,mice)
importFrom(moments,kurtosis)
importFrom(moments,skewness)
importFrom(nortest,ad.test)
importFrom(nortest,cvm.test)
importFrom(nortest,lillie.test)
importFrom(nortest,sf.test)
importFrom(plotly,ggplotly)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(purrr,imap_dfr)
importFrom(purrr,map_dfr)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,dnorm)
importFrom(stats,mahalanobis)
importFrom(stats,median)
importFrom(stats,pchisq)
importFrom(stats,plnorm)
importFrom(stats,pnorm)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,shapiro.test)
importFrom(stats,var)
importFrom(stringr,str_remove)
importFrom(stringr,str_to_title)
importFrom(stringr,str_to_upper)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,unnest)
importFrom(utils,head)
importFrom(viridis,viridis)
